@@ -8,15 +8,15 @@ from mautrix.util.async_db import Database
 from . import commands  # noqa: F401
 from .config import Config
 from .db import init as init_db, upgrade_table
-from .puppet import Puppet
-from .portal import Portal  # noqa: I100 (needs to be after because it relies on Puppet)
-from .user import User
-from .version import linkified_version, version
-from .web import ProvisioningAPI
 
 # This has to be imported after the rest of the modules because it relies on them
 # being imported already.
 from .matrix import MatrixHandler  # noqa: I100 I202
+from .portal import Portal  # noqa: I100 (needs to be after because it relies on Puppet)
+from .puppet import Puppet
+from .user import User
+from .version import linkified_version, version
+from .web import ProvisioningAPI
 
 
 class LinkedInBridge(Bridge):
@@ -24,7 +24,7 @@ class LinkedInBridge(Bridge):
     module = "linkedin_matrix"
     command = "linkedin-matrix"
     description = "A Matrix-LinkedIn Messages puppeting bridge."
-    repo_url = "https://gitlab.com/beeper/linkedin"
+    repo_url = "https://github.com/beeper/linkedin"
     version = version
     markdown_version = linkified_version
     config_class = Config
