@@ -1,5 +1,5 @@
-import os
 from typing import Any
+import os
 
 from mautrix.bridge.config import BaseBridgeConfig
 from mautrix.types import UserID
@@ -40,17 +40,13 @@ class Config(BaseBridgeConfig):
         copy("bridge.backfill.initial_limit")
         copy("bridge.backfill.invite_own_puppet")
         copy("bridge.backfill.missed_limit")
+        copy("bridge.backfill.unread_hours_threshold")
         copy("bridge.command_prefix")
         copy("bridge.delivery_receipts")
         copy("bridge.displayname_preference")
         copy("bridge.displayname_template")
         copy("bridge.double_puppet_allow_discovery")
         copy("bridge.double_puppet_server_map")
-        copy("bridge.encryption.allow")
-        copy("bridge.encryption.default")
-        copy("bridge.encryption.key_sharing.allow")
-        copy("bridge.encryption.key_sharing.require_cross_signing")
-        copy("bridge.encryption.key_sharing.require_verification")
         copy("bridge.federate_rooms")
         copy("bridge.initial_chat_sync")
         copy("bridge.invite_own_puppet_to_pm")
