@@ -1,9 +1,10 @@
+from typing import Optional, cast
 from datetime import datetime
-from typing import cast, Optional
 
 from asyncpg import Record
 from attr import dataclass
 from linkedin_messaging import URN
+
 from mautrix.types import EventID, RoomID
 
 from .model_base import Model
@@ -98,8 +99,8 @@ class Message(Model):
         li_receiver_urn: URN,
     ) -> Optional["Message"]:
         query = (
-            Message.select_constructor("li_thread_urn=$1 AND li_receiver_urn=$2 ")
-            + " ORDER BY timestamp DESC"
+            Message.select_constructor("li_thread_urn=$1 AND li_receiver_urn=$2")
+            + " ORDER BY timestamp DESC, index DESC"
             + " LIMIT 1"
         )
         row = await cls.db.fetchrow(
