@@ -1,11 +1,12 @@
-from typing import cast, Optional
+from typing import Optional, cast
 
 from asyncpg import Record
 from attr import dataclass
 from linkedin_messaging import URN
-from mautrix.types import ContentURI, SyncToken, UserID
 from yarl import URL
 
+from mautrix.types import ContentURI, SyncToken, UserID
+
 from .model_base import Model
 
 
