@@ -1,4 +1,5 @@
-from typing import cast, Optional, TYPE_CHECKING, Union
+from typing import TYPE_CHECKING, Optional, Union, cast
+import asyncio
 
 from mautrix.bridge import BaseMatrixHandler
 from mautrix.types import (
@@ -16,7 +17,7 @@ from mautrix.types.event.message import RelationType
 from mautrix.types.event.reaction import ReactionEventContent
 
 # these have to be in this particular order to avoid circular imports
-from . import user as u, portal as po  # noqa: I101
+from . import user as u, portal as po  # isort:skip
 
 if TYPE_CHECKING:
     from .__main__ import LinkedInBridge
@@ -111,8 +112,11 @@ class MatrixHandler(BaseMatrixHandler):
         portal: Optional[po.Portal] = await po.Portal.get_by_mxid(room_id)
         if not portal:
             return
-        # https://www.linkedin.com/voyager/api/messaging/conversations?action=typing
-        # {"conversationId":"2-ZmRhMGVmZDYtNDVjYS00Y2Y2LWE3ZTYtNmFkM2FlMGMxMDA1XzAxMg=="}
+
+        async def _send_typing(user_id: UserID):
+            await portal.handle_matrix_typing(await u.User.get_by_mxid(user_id))
+
+        await asyncio.gather(*(_send_typing(user_id) for user_id in typing))
 
     async def handle_ephemeral_event(
         self,
