@@ -1,10 +1,9 @@
-import asyncio
+from typing import TYPE_CHECKING, Any, AsyncGenerator, Optional, Union, cast
 from collections import deque
-from datetime import datetime
+from datetime import datetime, timedelta
 from io import BytesIO
-from typing import Any, AsyncGenerator, cast, Optional, TYPE_CHECKING, Union
+import asyncio
 
-import magic
 from linkedin_messaging import URN
 from linkedin_messaging.api_objects import (
     AttributedBody,
@@ -18,8 +17,10 @@ from linkedin_messaging.api_objects import (
     RealTimeEventStreamEvent,
     ThirdPartyMedia,
 )
+import magic
+
 from mautrix.appservice import IntentAPI
-from mautrix.bridge import async_getter_lock, BasePortal, NotificationDisabler
+from mautrix.bridge import BasePortal, NotificationDisabler, async_getter_lock
 from mautrix.errors import MatrixError, MForbidden
 from mautrix.types import (
     AudioInfo,
@@ -44,11 +45,7 @@ from mautrix.util.simple_lock import SimpleLock
 
 from . import puppet as p, user as u
 from .config import Config
-from .db import (
-    Message as DBMessage,
-    Portal as DBPortal,
-    Reaction as DBReaction,
-)
+from .db import Message as DBMessage, Portal as DBPortal, Reaction as DBReaction
 from .formatter import (
     linkedin_spinmail_to_matrix,
     linkedin_subject_to_matrix,
@@ -256,22 +253,23 @@ class Portal(DBPortal, BasePortal):
             Portal,
             await super().get_by_li_thread_urn(li_thread_urn, li_receiver_urn),
         )
-        if portal:
-            await portal.postinit()
-            return portal
+        if not portal:
+            if create:
+                portal = cls(
+                    li_thread_urn,
+                    li_receiver_urn=li_receiver_urn,
+                    li_is_group_chat=li_is_group_chat,
+                    li_other_user_urn=li_other_user_urn,
+                )
+                await portal.insert()
+            else:
+                return None
 
-        if create:
-            portal = cls(
-                li_thread_urn,
-                li_receiver_urn=li_receiver_urn,
-                li_is_group_chat=li_is_group_chat,
-                li_other_user_urn=li_other_user_urn,
-            )
-            await portal.insert()
-            await portal.postinit()
-            return portal
-
-        return None
+        if li_other_user_urn is not None:
+            portal.li_other_user_urn = li_other_user_urn
+        await portal.save()
+        await portal.postinit()
+        return portal
 
     @classmethod
     async def get_all_by_li_receiver_urn(
@@ -297,6 +295,11 @@ class Portal(DBPortal, BasePortal):
                 await portal.postinit()
                 yield portal
 
+    async def get_dm_puppet(self) -> Optional["p.Puppet"]:
+        if not self.is_direct:
+            return None
+        return await p.Puppet.get_by_li_member_urn(self.li_other_user_urn)
+
     # endregion
 
     # region Chat info updating
@@ -537,7 +540,7 @@ class Portal(DBPortal, BasePortal):
             initial_state.append(
                 {
                     "type": "m.room.encryption",
-                    "content": {"algorithm": "m.megolm.v1.aes-sha2"},
+                    "content": self.get_encryption_state_event_json(),
                 }
             )
             if self.is_direct:
@@ -716,6 +719,27 @@ class Portal(DBPortal, BasePortal):
                     conversation=conversation,
                 )
 
+            if (
+                conversation.unread_count == 0
+                or (
+                    (hours := self.config["bridge.backfill.unread_hours_threshold"]) > 0
+                    and conversation.last_activity_at
+                    and conversation.last_activity_at < datetime.now() - timedelta(hours=hours)
+                )
+            ) and (
+                most_recent := await DBMessage.get_most_recent(
+                    self.li_thread_urn, self.li_receiver_urn
+                )
+            ):
+                puppet = await source.bridge.get_double_puppet(source.mxid)
+                if puppet and puppet.is_real_user:
+                    if most_recent_reaction := await DBReaction.get_most_recent_by_li_message_urn(
+                        self.mxid, most_recent.li_message_urn
+                    ):
+                        await puppet.intent.mark_read(self.mxid, most_recent_reaction.mxid)
+                    else:
+                        await puppet.intent.mark_read(self.mxid, most_recent.mxid)
+
     async def _backfill(
         self,
         source: "u.User",
@@ -1001,7 +1025,7 @@ class Portal(DBPortal, BasePortal):
             await self._send_delivery_receipt(redaction_event_id)
 
     async def _handle_matrix_redaction(self, sender: "u.User", event_id: EventID):
-        if not self.mxid or not sender.client:
+        if not self.mxid or not sender.client or not sender.li_member_urn:
             return
 
         message = await DBMessage.get_by_mxid(event_id, self.mxid)
@@ -1064,6 +1088,9 @@ class Portal(DBPortal, BasePortal):
                 ).insert()
                 await self._send_delivery_receipt(event_id)
 
+    async def handle_matrix_typing(self, source: "u.User"):
+        await source.client.set_typing(self.li_thread_urn)
+
     # endregion
 
     # region LinkedIn event handling
@@ -1094,10 +1121,7 @@ class Portal(DBPortal, BasePortal):
         return True
 
     async def handle_linkedin_message(
-        self,
-        source: "u.User",
-        sender: "p.Puppet",
-        message: ConversationEvent,
+        self, source: "u.User", sender: "p.Puppet", message: ConversationEvent
     ):
         try:
             if message.subtype == "CONVERSATION_UPDATE":
@@ -1116,10 +1140,7 @@ class Portal(DBPortal, BasePortal):
             self.log.exception(f"Error handling LinkedIn message {message.entity_urn}: {e}")
 
     async def _handle_linkedin_message(
-        self,
-        source: "u.User",
-        sender: "p.Puppet",
-        message: ConversationEvent,
+        self, source: "u.User", sender: "p.Puppet", message: ConversationEvent
     ):
         assert self.mxid
         assert self.li_receiver_urn
@@ -1274,6 +1295,7 @@ class Portal(DBPortal, BasePortal):
                 source,
                 reaction_event_id,
                 reaction_summary,
+                message.created_at,
             )
 
     async def _handle_linkedin_message_deletion(
@@ -1306,6 +1328,7 @@ class Portal(DBPortal, BasePortal):
         source: "u.User",
         reaction_event_id: EventID,
         reaction_summary: ReactionSummary,
+        timestamp: Optional[datetime],
     ) -> list[EventID]:
         if not reaction_summary.emoji or not source.client:
             return []
@@ -1321,7 +1344,9 @@ class Portal(DBPortal, BasePortal):
             sender = await p.Puppet.get_by_li_member_urn(reactor.reactor_urn)
             intent = sender.intent_for(self)
 
-            mxid = await intent.react(self.mxid, reaction_event_id, reaction_summary.emoji)
+            mxid = await intent.react(
+                self.mxid, reaction_event_id, reaction_summary.emoji, timestamp=timestamp
+            )
             mxids.append(mxid)
 
             self.log.debug(
@@ -1466,16 +1491,18 @@ class Portal(DBPortal, BasePortal):
             file_data, decryption_info = encrypt_attachment(file_data)
             upload_mime_type = "application/octet-stream"
             filename = None
-        url = await intent.upload_media(file_data, mime_type=upload_mime_type, filename=filename)
+        url = await intent.upload_media(
+            file_data,
+            mime_type=upload_mime_type,
+            filename=filename,
+            async_upload=cls.config["homeserver.async_media"],
+        )
         if decryption_info:
             decryption_info.url = url
         return url, info, decryption_info
 
     async def handle_linkedin_reaction_add(
-        self,
-        source: "u.User",
-        sender: "p.Puppet",
-        event: RealTimeEventStreamEvent,
+        self, source: "u.User", sender: "p.Puppet", event: RealTimeEventStreamEvent
     ):
         if not event.event_urn or not self.li_receiver_urn or not event.reaction_summary:
             return
@@ -1525,10 +1552,7 @@ class Portal(DBPortal, BasePortal):
         self._dedup.remove(dedup_id)
 
     async def handle_linkedin_reaction_remove(
-        self,
-        source: "u.User",
-        sender: "p.Puppet",
-        event: RealTimeEventStreamEvent,
+        self, source: "u.User", sender: "p.Puppet", event: RealTimeEventStreamEvent
     ):
         if (
             not self.mxid
@@ -1550,4 +1574,24 @@ class Portal(DBPortal, BasePortal):
                 await self.main_intent.redact(reaction.mx_room, reaction.mxid)
             await reaction.delete()
 
+    async def handle_linkedin_conversation_read(self, source: "u.User"):
+        most_recent = await DBMessage.get_most_recent(self.li_thread_urn, self.li_receiver_urn)
+        if not most_recent:
+            return
+        puppet = await source.bridge.get_double_puppet(source.mxid)
+        if puppet and puppet.is_real_user:
+            await puppet.intent.mark_read(self.mxid, most_recent.mxid)
+
+    async def handle_linkedin_seen_receipt(
+        self, source: "u.User", sender: "p.Puppet", event: RealTimeEventStreamEvent
+    ):
+        if messages := await DBMessage.get_all_by_li_message_urn(
+            event.seen_receipt.event_urn, self.li_receiver_urn
+        ):
+            messages.sort(key=lambda m: m.index)
+            await sender.intent.mark_read(self.mxid, messages[-1].mxid)
+
+    async def handle_linkedin_typing(self, sender: "p.Puppet"):
+        await sender.intent.set_typing(self.mxid)
+
     # endregion
