@@ -1,19 +1,11 @@
-import asyncio
-import time
+from typing import TYPE_CHECKING, AsyncGenerator, AsyncIterable, Awaitable, Optional, cast
 from asyncio.futures import Future
 from datetime import datetime
-from typing import (
-    AsyncGenerator,
-    AsyncIterable,
-    Awaitable,
-    cast,
-    Optional,
-    TYPE_CHECKING,
-)
+import asyncio
+import time
 
-import aiohttp
 from aiohttp.client_exceptions import TooManyRedirects
-from linkedin_messaging import LinkedInMessaging, URN
+from linkedin_messaging import URN, LinkedInMessaging
 from linkedin_messaging.api_objects import (
     Conversation,
     ConversationEvent,
@@ -21,17 +13,13 @@ from linkedin_messaging.api_objects import (
     RealTimeEventStreamEvent,
     UserProfileResponse,
 )
-from mautrix.bridge import async_getter_lock, BaseUser
+import aiohttp
+
+from mautrix.bridge import BaseUser, async_getter_lock
 from mautrix.errors import MNotFound
-from mautrix.types import (
-    PushActionType,
-    PushRuleKind,
-    PushRuleScope,
-    RoomID,
-    UserID,
-)
+from mautrix.types import PushActionType, PushRuleKind, PushRuleScope, RoomID, UserID
 from mautrix.util.bridge_state import BridgeState, BridgeStateEvent
-from mautrix.util.opt_prometheus import async_time, Gauge, Summary
+from mautrix.util.opt_prometheus import Gauge, Summary, async_time
 from mautrix.util.simple_lock import SimpleLock
 
 from . import portal as po, puppet as pu
@@ -181,6 +169,18 @@ class User(DBUser, BaseUser):
 
         return None
 
+    async def get_puppet(self) -> Optional["pu.Puppet"]:
+        if not self.li_member_urn:
+            return None
+        return await pu.Puppet.get_by_li_member_urn(self.li_member_urn)
+
+    async def get_portal_with(
+        self, puppet: "pu.Puppet", create: bool = True
+    ) -> Optional[po.Portal]:
+        # We should probably make this work eventually, but for now, creating chats will just not
+        # work.
+        return None
+
     # endregion
 
     # region Session Management
@@ -479,14 +479,16 @@ class User(DBUser, BaseUser):
             self.client.add_event_listener("STREAM_ERROR", self.handle_linkedin_listener_error)
             self.client.add_event_listener("event", self.handle_linkedin_event)
             self.client.add_event_listener("reactionAdded", self.handle_linkedin_reaction_added)
+            self.client.add_event_listener("action", self.handle_linkedin_action)
+            self.client.add_event_listener("fromEntity", self.handle_linkedin_from_entity)
             self.listener_event_handlers_created = True
         await self.client.start_listener()
 
     _prev_connected_bridge_state = -600
 
     async def _push_connected_state(self):
-        # Only send once every 10 minutes.
-        if self._prev_connected_bridge_state + 600 < time.monotonic():
+        # Only send once every 12 hours.
+        if self._prev_connected_bridge_state + (12 * 60 * 60) < time.monotonic():
             await self.push_bridge_state(BridgeStateEvent.CONNECTED)
             self._prev_connected_bridge_state = time.monotonic()
         else:
@@ -539,9 +541,7 @@ class User(DBUser, BaseUser):
             raise Exception("Invalid sender: no entity_urn found!", event)
 
         portal = await po.Portal.get_by_li_thread_urn(
-            thread_urn,
-            li_receiver_urn=self.li_member_urn,
-            create=False,
+            thread_urn, li_receiver_urn=self.li_member_urn, create=False
         )
         if not portal:
             conversations = await self.client.get_conversations()
@@ -568,8 +568,19 @@ class User(DBUser, BaseUser):
         thread_urn, message_urn = map(URN, event.event_urn.id_parts)
 
         portal = await po.Portal.get_by_li_thread_urn(
-            thread_urn, li_receiver_urn=self.li_member_urn
+            thread_urn, li_receiver_urn=self.li_member_urn, create=False
         )
+        if not portal:
+            conversations = await self.client.get_conversations()
+            for conversation in conversations.elements:
+                if conversation.entity_urn == thread_urn:
+                    await self._sync_thread(conversation)
+                    break
+
+            # Nothing more to do, since the backfill should handle the message coming
+            # in.
+            return
+
         puppet = await pu.Puppet.get_by_li_member_urn(event.actor_mini_profile_urn)
 
         await portal.backfill_lock.wait(message_urn)
@@ -578,4 +589,34 @@ class User(DBUser, BaseUser):
         else:
             await portal.handle_linkedin_reaction_remove(self, puppet, event)
 
+    async def handle_linkedin_action(self, event: RealTimeEventStreamEvent):
+        if event.action != "UPDATE":
+            return
+        if (
+            (raw_conversation := event.conversation)
+            and isinstance(raw_conversation, dict)
+            and (conversation := Conversation.from_dict(raw_conversation))
+            and conversation.read
+        ):
+            if portal := await po.Portal.get_by_li_thread_urn(
+                conversation.entity_urn, li_receiver_urn=self.li_member_urn, create=False
+            ):
+                await portal.handle_linkedin_conversation_read(self)
+
+    async def handle_linkedin_from_entity(self, event: RealTimeEventStreamEvent):
+        if seen_receipt := event.seen_receipt:
+            conversation_urn = URN(seen_receipt.event_urn.id_parts[0])
+            if portal := await po.Portal.get_by_li_thread_urn(
+                conversation_urn, li_receiver_urn=self.li_member_urn, create=False
+            ):
+                puppet = await pu.Puppet.get_by_li_member_urn(event.from_entity)
+                await portal.handle_linkedin_seen_receipt(self, puppet, event)
+
+        if isinstance(event.conversation, str):
+            if portal := await po.Portal.get_by_li_thread_urn(
+                URN(event.conversation), li_receiver_urn=self.li_member_urn, create=False
+            ):
+                puppet = await pu.Puppet.get_by_li_member_urn(event.from_entity)
+                await portal.handle_linkedin_typing(puppet)
+
     # endregion
